# Implements a 2-button remote control on a Pico with auto repeat.
# Load libraries
from machine import Pin
import uasyncio as asyncio
from primitives.switch import Switch
from primitives.delay_ms import Delay_ms

# Import all implemented classes
from ir_tx.nec import NEC
from ir_tx.sony import SONY_12, SONY_15, SONY_20
from ir_tx.philips import RC5, RC6_M0

######## **** DISPLAY GREETING **** ############################
s = '''Test for IR transmitter.
test() for NEC protocol
test(1) for Sony SIRC 12 bit
test(2) for Sony SIRC 15 bit
test(3) for Sony SIRC 20 bit
test(4) for Philips RC-5 protocol
test(5) for Philips RC-6 mode 0.
'''

srp2 = '''
IR LED gate on pin 17
Connect pin 18 to a ground pin to send addr 1 data 54
Connect pin 19 to a ground pin to send addr 0x10 data 0x34.'''
################################################################

loop = asyncio.get_event_loop()

# If the "button" is held down the normal behaviour is to
# retransmit the data but most NEC models send a REPEAT code
class Rbutton:
    toggle = 1  # toggle is ignored in NEC mode
    def __init__(self, irb, pin, addr, data, proto):
        self.irb = irb
        self.sw = Switch(pin)
        self.addr = addr
        self.data = data
        self.proto = proto

        self.sw.close_func(self.cfunc)
        self.sw.open_func(self.ofunc)
        self.tim = Delay_ms(self.repeat)

    def cfunc(self):  # Button push: send data
        tog = 0 if self.proto < 3 else Rbutton.toggle  # NEC, sony 12, 15: toggle==0
        self.irb.transmit(self.addr, self.data, tog, True)  # Test validation
        # Auto repeat. The Sony protocol specifies 45ms but this is tight.
        # In 20 bit mode a data burst can be upto 39ms long.
        self.tim.trigger(108)

    def ofunc(self):  # Button release: cancel repeat timer
        self.tim.stop()
        Rbutton.toggle ^= 1  # Toggle control

    async def repeat(self):
        await asyncio.sleep(0)  # Let timer stop before retriggering
        if not self.sw():  # Button is still pressed: retrigger
            self.tim.trigger(108)
            if self.proto == 0:
                self.irb.repeat()  # NEC special case: send REPEAT code
            else:
                tog = 0 if self.proto < 3 else Rbutton.toggle  # NEC, sony 12, 15: toggle==0
                self.irb.transmit(self.addr, self.data, tog, True)  # Test validation

async def main(proto):
    # Test uses a 38KHz carrier.
    # Pin for the IR LED gate
    pin = Pin(17, Pin.OUT, value = 0)
    # List with all possible protocols
    classes = (NEC, SONY_12, SONY_15, SONY_20, RC5, RC6_M0)
    irb = classes[proto](pin, 38000)
    # Uncomment the following to print transmition timing
    # irb.timeit = True

    b = []  # Rbutton instances
    px3 = Pin(18, Pin.IN, Pin.PULL_UP)
    px4 = Pin(19, Pin.IN, Pin.PULL_UP)
    # edit these two for the desired effect/data/comand #
    b.append(Rbutton(irb, px3, 0x1, 0x36, proto))
    b.append(Rbutton(irb, px4, 0x10, 0x34, proto))
    #####################################################
    led = Pin(25, Pin.OUT)
    while True:
        await asyncio.sleep_ms(500)  # Obligatory flashing LED.
        led(not led())

# Function to be called to start the sending of the IR Data
def SendIRData(proto=0):
    loop.run_until_complete(main(proto))

print(''.join((s, srp2)))

# Endless loop for continuous operation
while True:
    SendIRData()